from __future__ import annotations

import math
from typing import TYPE_CHECKING, Literal

from prettyqt import constants, core
from prettyqt.qt import QtGui
from prettyqt.utils import bidict, datatypes


if TYPE_CHECKING:
    from collections.abc import Iterator


ElementTypeStr = Literal[
    "move_to_element", "line_to_element", "curve_to_element", "curve_to_data_element"
]

ELEMENT_TYPES: bidict[ElementTypeStr, QtGui.QPainterPath.ElementType] = bidict(
    move_to_element=QtGui.QPainterPath.ElementType.MoveToElement,
    line_to_element=QtGui.QPainterPath.ElementType.LineToElement,
    curve_to_element=QtGui.QPainterPath.ElementType.CurveToElement,
    curve_to_data_element=QtGui.QPainterPath.ElementType.CurveToDataElement,
)


class PainterPath(QtGui.QPainterPath):
    def __len__(self):
        return self.elementCount()

    def __getitem__(self, index: int) -> QtGui.QPainterPath.Element:
        return self.elementAt(index)

    def __iter__(self) -> Iterator[QtGui.QPainterPath.Element]:
        return iter(self.elementAt(i) for i in range(self.elementCount()))

    def __setitem__(self, index: int, value: tuple[int, int]):
        self.setElementPositionAt(index, *value)

    def __iadd__(self, other):
        if not isinstance(
            other,
            core.QPoint | core.QRect | QtGui.QPainterPath | QtGui.QRegion,
        ):
            raise TypeError(other)
        self.add(other)
        return self

    def add(self, other: core.QPoint | core.QRect | QtGui.QPainterPath | QtGui.QRegion):
        match other:
            case QtGui.QPolygonF():
                self.addPolygon(other)
            case QtGui.QPainterPath():
                self.addPath(other)
            case core.QRect():
                self.addRect(other)
            case QtGui.QRegion():
                self.addRegion(other)

    def __bool__(self):
        return not self.isEmpty()

    def __contains__(self, item: core.QPointF | core.QRectF | QtGui.QPainterPath):
        return self.contains(item)

    def add_rect(self, rect: datatypes.RectType | datatypes.RectFType):
        self.addRect(datatypes.to_rectf(rect))

    def set_fill_rule(self, rule: constants.FillRuleStr):
        """Set fill rule.

        Args:
            rule: fill rule to use
        """
        self.setFillRule(constants.FILL_RULE[rule])

    def get_fill_rule(self) -> constants.FillRuleStr:
        """Return current fill rule.

        Returns:
            fill rule
        """
        return constants.FILL_RULE.inverse[self.fillRule()]

    def get_bounding_rect(self) -> core.RectF:
        return core.RectF(self.boundingRect())

    def get_simplified(self) -> PainterPath:
        return PainterPath(self.simplified())

    def to_reversed(self) -> PainterPath:
        return PainterPath(self.toReversed())

    @classmethod
    def create_star(cls, size: int):
        path = cls()
        star_center_x = size / 2
        star_center_y = size / 2
        radius_outer = size * 0.35
        golden_ratio = (1 + math.sqrt(5)) / 2
        radius_inner = radius_outer / (1 + golden_ratio)
        theta_start = math.pi / 2
        theta_inc = (2 * math.pi) / 10
        for n in range(11):
            theta = theta_start + (n * theta_inc)
            theta = theta % (2 * math.pi)
            if n % 2 == 0:
                x = radius_outer * math.cos(theta)
                y = radius_outer * math.sin(theta)

            else:
                x = radius_inner * math.cos(theta)
                y = radius_inner * math.sin(theta)

            x_adj = star_center_x - x
            y_adj = star_center_y - y + 3
            if n == 0:
                path.moveTo(x_adj, y_adj)
            else:
                path.lineTo(x_adj, y_adj)

        return path


if __name__ == "__main__":
    from prettyqt import gui, widgets

    app = widgets.app()

    class Test(widgets.Widget):
        def paintEvent(self, event):
            with gui.Painter(self) as painter:
                painter.drawPath(gui.PainterPath.create_star(400, 200))

    p = PainterPath.create_star(400, 200)
    p += core.QPoint(1, 1)

    t = Test()
    t.show()
    app.exec()



"""The charts module provides a set of easy-to-use chart components."""

from __future__ import annotations

from prettyqt.qt.QtCharts import *  # noqa: F403

from .barset import BarSet
from .boxset import BoxSet
from .candlestickset import CandlestickSet
from .pieslice import PieSlice
from .abstractaxis import AbstractAxis, AbstractAxisMixin
from .valueaxis import ValueAxis, ValueAxisMixin
from .datetimeaxis import DateTimeAxis
from .categoryaxis import CategoryAxis
from .logvalueaxis import LogValueAxis
from .barcategoryaxis import BarCategoryAxis
from .abstractseries import AbstractSeries, AbstractSeriesMixin
from .abstractbarseries import AbstractBarSeries, AbstractBarSeriesMixin
from .barseries import BarSeries
from .percentbarseries import PercentBarSeries
from .horizontalbarseries import HorizontalBarSeries
from .horizontalpercentbarseries import HorizontalPercentBarSeries
from .stackedbarseries import StackedBarSeries
from .horizontalstackedbarseries import HorizontalStackedBarSeries
from .xyseries import XYSeries, XYSeriesMixin
from .scatterseries import ScatterSeries
from .lineseries import LineSeries
from .legend import Legend
from .chart import Chart, ChartMixin
from .polarchart import PolarChart
from .chartview import ChartView
from prettyqt.qt import QtCharts

QT_MODULE = QtCharts

__all__ = [
    "AbstractAxis",
    "AbstractAxisMixin",
    "AbstractBarSeries",
    "AbstractBarSeriesMixin",
    "AbstractSeries",
    "AbstractSeriesMixin",
    "BarCategoryAxis",
    "BarSeries",
    "BarSet",
    "BoxSet",
    "CandlestickSet",
    "CategoryAxis",
    "Chart",
    "ChartMixin",
    "ChartView",
    "DateTimeAxis",
    "HorizontalBarSeries",
    "HorizontalPercentBarSeries",
    "HorizontalStackedBarSeries",
    "Legend",
    "LineSeries",
    "LogValueAxis",
    "PercentBarSeries",
    "PieSlice",
    "PolarChart",
    "ScatterSeries",
    "StackedBarSeries",
    "ValueAxis",
    "ValueAxisMixin",
    "XYSeries",
    "XYSeriesMixin",
]



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, ClassVar

import mknodes

from prettyqt import constants, itemmodels
from prettyqt.utils import helpers


if TYPE_CHECKING:
    from prettyqt import core


logger = logging.getLogger(__name__)


class MkNodesModel(itemmodels.TreeModel):
    """An ItemModel for displaying a mknodes tree."""

    HEADER: ClassVar = ["Repr", "Type", "Files"]

    class Roles:
        MarkdownRole = constants.USER_ROLE + 5325

    def headerData(
        self,
        section: int,
        orientation: constants.Orientation,
        role: constants.ItemDataRole = constants.DISPLAY_ROLE,
    ):
        match orientation, role, section:
            case constants.HORIZONTAL, constants.DISPLAY_ROLE, _:
                return self.HEADER[section]
        return None

    def columnCount(self, index: core.ModelIndex | None = None) -> int:
        return len(self.HEADER)

    def data(
        self,
        index: core.ModelIndex,
        role: constants.ItemDataRole = constants.DISPLAY_ROLE,
    ):
        node = self.data_by_index(index).obj
        match role, index.column():
            case constants.DISPLAY_ROLE, 0:
                match node:
                    case mknodes.MkNav():
                        return f"{node.title}/"
                    case mknodes.MkPage():
                        return node.path
                    case _:
                        return repr(node)
            # case constants.DISPLAY_ROLE, 1:
            #     with contextlib.suppress(AttributeError):
            #         return node._to_markdown().count("\n")
            case constants.DISPLAY_ROLE, 1:
                return type(node).__name__
            case constants.DISPLAY_ROLE, 2:
                keys = node.get_resources().keys()
                return ", ".join(keys)
            case self.Roles.MarkdownRole, _:
                return node.to_markdown()

    def _fetch_object_children(self, item) -> list[MkNodesModel.TreeItem]:
        return [self.TreeItem(i) for i in item.obj.children]

    def _has_children(self, item) -> bool:
        return len(item.obj.children) > 0


if __name__ == "__main__":
    page = mknodes.MkPage()
    page += mknodes.MkAdmonition("test")
    data = dict(a=["1", "2"], b=["c", "D"])
    page += mknodes.MkTable(data=data)
    page += mknodes.MkDocStrings(helpers, header="DocStrings")
    model = MkNodesModel(page)
    print(page.to_markdown())
